#!/usr/bin/perl

#
# Bö -- A footnote extension for Markdown.
# Version 1.0b2, 2004-08-26
#
# Copyright (c) 2004 Olivier Scherler
# Movable Type plugin installation code stolen from John Gruber,
# without permission yet.
#
# Special thanks to:
#
# - [John Gruber][] for writing Markdown, Smarty Pants and Daring Fireball;
# - [jr][], for his invaluable opinion and the WordPress conversion;
# - [Timothy Appnel][], for his patient answers on the Movable Type Developer Forum.
#
# [John Gruber]: http://daringfireball.net/
# [jr]: http://mycvs.org/
# [Timothy Appnel]: http://www.timaoutloud.org/
# 
# To do:
#	- require Markdown? (I mean, in a Perl statement)
#


package Bo;
require 5.006_000;
use strict;
use warnings;

use utf8;
use vars qw($VERSION);
$VERSION = '1.0b2';
# Thu 26 Aug 2004

#
# Global default settings:
#
my $g_tab_width = 4;	# Tab width
my $g_indent = ' ' x $g_tab_width;

#
# Globals
#
my @g_text = ();		# Array with the text to allow two-pass processing.
my $g_out_text = '';	# Output text.
my @g_labels = ();		# Order in which the labels appear. Duplicates are ignored.
my %g_labels = ();		# Hash with the footnote number for each label.
my %g_notes = ();		# Hash with the footnotes text.
my $g_note_num = 0;		# Current footnote number.

my $g_in_note = 0;		# Are we processing a footnote?
my $g_label = '';		# The label of the current footnote.
my $g_note = '';		# The footnote text.

my $g_id_prefix = '';	# Prefix for HTML id's

#
# HTML templates:
#

# Footnote reference
my $g_ref_template = qq(<sup><a id="ref_{PREFIX}{LABEL}" href="#def_{PREFIX}{LABEL}">[{NUM}]</a></sup>);

# Footnote definition label
my $g_def_template = qq(<sup><a id="def_{PREFIX}{LABEL}" href="#ref_{PREFIX}{LABEL}">[{NUM}]</a></sup> {NOTE}\n\n);

#
# Global regexes:
#
my $g_label_regex = qq{[A-Za-z0-9_-]+};					# Label
my $g_ref_regex = qq{\\#\\[($g_label_regex)\\]};		# Footnote reference, $1 = label
my $g_def_regex = qq{^\\[($g_label_regex)\\]:\\s*};		# Footnote definition, $1 = label

#### Movable Type plug-in interface #####################################

eval {require MT};  # Test to see if we're running in MT.
unless ($@)
{
    require MT;
    import  MT;
    require MT::Template::Context;
    import  MT::Template::Context;

    my $filters = MT->all_text_filters();
    my $markdown_ref;
    my $markdown_sp_ref;

	if(exists( $filters->{'markdown'} ) )
		{ $markdown_ref = $filters->{'markdown'}{on_format}; }

	if(exists( $filters->{'markdown_with_smartypants'} ) )
		{ $markdown_sp_ref = $filters->{'markdown_with_smartypants'}{on_format}; }

	if( $markdown_ref )
	{
		# using bo_with_markdown and bo_with_markdown_and_smartypants doesn't work
		# because MT apparently truncates the identifiers to 16 characters (or maybe even less).
		MT->add_text_filter( 'bo_with_md' => {
			label     => 'B&ouml; with Markdown',
			docs      => '',
			on_format => sub
			{
				my $text = shift;
				my $ctx  = shift;
				$text = Bo( $text, $ctx );
				$text = &$markdown_ref( $text );
			},
		});
	}

	if( $markdown_sp_ref )
	{
		MT->add_text_filter( 'bo_with_mdsp' => {
			label     => 'B&ouml; with Markdown and SmartyPants',
			docs      => '',
			on_format => sub {
				my $text = shift;
				my $ctx  = shift;
				$text = Bo( $text, $ctx );
				$text = &$markdown_sp_ref( $text );
			},
		});
	}
}


#
# Main function.
#
sub Bo
{
	my $text = shift || return '';
	my $ctx = shift;

	_BuildPrefix( $ctx );
	_PrepareText( $text );
	_ListFootnoteCalls();
	_ExtractFootnotes();
	_PrintFootnotes();

	return $g_out_text . "\n";
}


#
# _BuildPrefix: build a prefix for HTML id's out Movable Type's blog
# and entry IDs if available.
#
sub _BuildPrefix
{
	my $ctx = shift;

	my $blog_id = '';
	my $entry_id = '';

	my $entry = $ctx->stash('entry') if( $ctx );
	if( $entry )
	{
		$blog_id = $entry->blog_id;
		$entry_id = $entry->id;
	}
	
	$g_id_prefix = '';
	
	$g_id_prefix .= $blog_id . '_' if( $blog_id );
	$g_id_prefix .= $entry_id . '_' if( $entry_id );
}


#
# _PrepareText: standardise line endings, detab and split lines into array.
#
sub _PrepareText
{
	my $text = shift || return '';

	@g_text = ();			# Array with the text to allow two-pass processing.
	@g_labels = ();			# Order in which the labels appear. Duplicates are ignored.
	%g_labels = ();			# Hash with the footnote number for each label.
	%g_notes = ();			# Hash with the footnotes text.
	$g_note_num = 0;		# Current footnote number.

	# Standardize line endings:
	$text =~ s{\r\n}{\n}g; 	# DOS to Unix
	$text =~ s{\r}{\n}g; 	# Mac to Unix

	# Make sure $text ends with a couple of newlines:
	$text .= "\n\n";

	# Convert all tabs to spaces.
	$text = _Detab( $text );

	@g_text = split( /\n/, $text );
}


#
# _ListFootnoteCalls: store the footnote call order and number.
#
sub _ListFootnoteCalls
{
	# First pass, list footnote labels in the order in which they appear in the text.

	my $line;

	foreach $line( @g_text )
	{
		# Replace footnote labels with a link.
		$line =~ s/$g_ref_regex/
			my $label = lc $1;
		
			if( not defined( $g_labels{$label} ) )
			{
				# New label, increase the footnote number
				$g_note_num++;
				# Add it to the list
				push( @g_labels, $label );
				# Store the footnote number
				$g_labels{$label} = $g_note_num;
			}
		
			# Lookup the footnote number and make the link
			my $num = $g_labels{$label};
			
			my $link = $g_ref_template;

			$link =~ s|{PREFIX}|$g_id_prefix|g;
			$link =~ s|{LABEL}|$label|g;
			$link =~ s|{NUM}|$num|g;

			$link;
		/gex;
	}
}


#
# _ExtractFootnotes: remove footnote bodies from the text and store them in a hash.
#
sub _ExtractFootnotes
{
	# Second pass, store the actual footnotes and remove them from the text.
	
	$g_out_text = '';
	
	$g_in_note = 0;		# Are we processing a footnote?
	$g_label = '';		# The label of the current footnote.
	$g_note = '';		# The footnote text.

	my $line;

	foreach $line( @g_text )
	{
		$line .= "\n";
	
		# No else because _InFootnote modifies $g_in_note
		_InFootnote( $line ) if( $g_in_note == 1 );
		_OutOfFootnote( $line ) if( $g_in_note == 0 );
	}
	
	_AddFootnote() if( $g_in_note );
}


#
# _InFootnote: process a line when we're inside (or just exiting) a footnote.
#
sub _InFootnote #( $line )
{
	my $line = shift;

	# If the line is indented, it's part of the fotnote.
	# Remove one indentation level and append it to the footnote text.

	# The line is part of the footnote if it's empty or is indented by at least one level.
	if( $line =~ m/^\s*$/ or $line =~ m/$g_indent/ )
	{
		# Reduce intentation by one level
		$line =~ s/^$g_indent//;
		# Append to the footnote text
		$g_note .= $line;
	}
	else
	{
		# The footnote ends here.
		_AddFootnote();
	}
}


#
# _OutOfFootnote: process a line when we're outside of (or about to enter) a footnote.
#
sub _OutOfFootnote #( $line )
{
	my $line = shift;

	# We are not currently processing a footnote.
	
	# If the line starts by a known footnote label, start processing the footnote
	if( $line =~ m/$g_def_regex/ and $g_labels{lc $1} )
	{
		# Remove the label
		$line =~ s/$g_def_regex//;

		# Enter footnote mode, keep track of the label and store the text.
		$g_in_note = 1;
		$g_label = lc $1;
		$g_note .= $line;
	}
	else
	{
		# If the line does not start by a known footnote label, output it as-is.
		$g_out_text .= $line;
	}
}


#
# _AddFootnote: add the current footnoote to the hash.
#
sub _AddFootnote
{
	# Store the footnote text
	$g_notes{ $g_label } = $g_note;

	# Reset the current footnote.
	$g_in_note = 0;
	$g_label = '';
	$g_note = '';
}


#
# _PrintFootnotes: print the footnote bodies in the right order.
#
sub _PrintFootnotes
{
	# Third pass, print the footnotes
	
	$g_out_text .= "\n";

	my $label;
	
	# @g_labels contains the footnote labels in the right order and without duplicates.
	foreach $label( @g_labels )
	{
		my $num = $g_labels{$label};
		my $note = $g_notes{$label};
		
		my $link = $g_def_template;
		
		$link =~ s|{PREFIX}|$g_id_prefix|g;
		$link =~ s|{LABEL}|$label|g;
		$link =~ s|{NUM}|$num|g;
		$link =~ s|{NOTE}|$note|g;
		
		$g_out_text .= $link;
	}
}


#
# _Detab: replace spaces with tabs.
# Cribbed from John Gruber's Markdown, who cribbed it from a post by Bart Lateur:
# <http://www.nntp.perl.org/group/perl.macperl.anyperl/154>
#
sub _Detab
{
	my $text = shift || return '';

	$text =~ s{(.*?)\t}{$1.(' ' x ($g_tab_width - length($1) % $g_tab_width))}ge;
	return $text;
}


1;

__END__


NAME

	Bö


DESCRIPTION

Bö is a preprocessor for John Gruber's Markdown, a text-to-HTML filter that translates an
easy-to-read / easy-to-write structured text format into HTML.

Its primary (and currently only) purpose is to add smart footnote functionality to
Markdown.


SYNTAX

Footnotes are called from the text like this:

	#[label]

Where label may contain A-Z, a-z, _ and -.

The footnote bodies can be placed anywhere in the text. They will be automatically moved
to the end and sorted in cite order.

	[label]: text
	
		Second paragraph

To indicate that a footnote has several paragraphs, the text must be indented by one tab
or four spaces. The footnote body will be processed my Markdown after it's indententing
level has been decreased by one step and can therefore contain any valid Markdown.


BUGS

We don't do bugs. No way, nada, never.

In the very unlikely case of Bö doing something you don't want it to do, contact us and
if we like your tone, we may consider changing Bö's behaviour.


VERSION HISTORY

    1.0b2: Thu Aug 26, 2004

        First public release.


AUTHOR

    Olivier Scherler
    http://ithink.ch/blog/


ADDITIONAL CREDITS

* John Gruber for writing Markdown, Smarty Pants and Daring Fireball;

	http://daringfireball.net/

* jr, for his invaluable opinion and the WordPress conversion

	http://mycvs.org/

* Timothy Appnel, for his patient answers on the Movable Type Developer Forum.

	http://www.timaoutloud.org/


COPYRIGHT AND LICENSE

    Copyright (c) 2004 Olivier Scherler
    (http://ithink.ch/blog/)
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

*   Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

*   Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

*   Neither the name "Bö" nor the names of its contributors may be used to
	endorse or promote products derived from this software without specific
	prior written permission.

This software is provided by the copyright holders and contributors "as is"
and any express or implied warranties, including, but not limited to, the 
implied warranties of merchantability and fitness for a particular purpose 
are disclaimed. In no event shall the copyright owner or contributors be 
liable for any direct, indirect, incidental, special, exemplary, or 
consequential damages (including, but not limited to, procurement of 
substitute goods or services; loss of use, data, or profits; or business 
interruption) however caused and on any theory of liability, whether in 
contract, strict liability, or tort (including negligence or otherwise) 
arising in any way out of the use of this software, even if advised of the
possibility of such damage.
